@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(IO::Async::Notifier);
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 =head1 NAME
 
@@ -561,7 +561,7 @@ reused in preference to handing out a new ID.
 sub next_channel {
 	my $self = shift;
 	$self->{channel} //= 0;
-	return shift @{$self->{available_channel_id}} if @{$self->{available_channel_id} ||=[] };
+	return shift @{$self->{available_channel_id}} if @{$self->{available_channel_id} ||= [] };
 	return undef if $self->{channel} >= $self->channel_max;
 	++$self->{channel}
 }
@@ -609,8 +609,14 @@ Returns the new L<Net::Async::AMQP::Channel> instance.
 sub open_channel {
 	my $self = shift;
 	my %args = @_;
-	my $channel = $args{channel} // $self->next_channel;
-	die "Channel " . $channel . " exists already" if exists $self->{channel_map}{$channel};
+	my $channel;
+	if($args{channel}) {
+		$channel = delete $args{channel};
+		extract_by { $channel == $_ } @{$self->{available_channel_id}} if exists $self->{available_channel_id};
+	} else {
+		$channel = $self->next_channel;
+	}
+	die "Channel " . $channel . " exists already: " . $self->{channel_map}{$channel} if exists $self->{channel_map}{$channel};
 	my $c = $self->create_channel($channel);
 	my $f = $c->future;
 
@@ -1056,6 +1062,7 @@ Returns $self.
 
 sub process_frame {
 	my ($self, $frame) = @_;
+	$self->debug_printf("Received %s", amqp_frame_info($frame));
 
 	my $frame_type = amqp_frame_type($frame);
 
@@ -1209,6 +1216,8 @@ sub send_frame {
 
 	$frame->channel($args{channel} // 0) unless defined $frame->channel;
 
+	$self->debug_printf("Sending %s", amqp_frame_info($frame));
+
 	# Get bytes to send across our transport
 	my $data = $frame->to_raw_frame;
 
